import { LkrOptions } from './Contracts';
/**
 * A Fluent Storage API.
 */
declare class Lkr {
    private opts;
    private store;
    /**
     * Create a Lkr instance.
     *
     * @param   {Object}  options               The configuration options
     * @param   {Object}  options.drivers       The storage drivers
     * @param   {String}  options.driver        The default storage driver
     * @param   {String}  options.namespace     The namespace
     * @param   {String}  options.separator     The string that separates the namespace and key
     * @param   {Object}  [options.serializer]  The serializer to use
     *
     * @return  {void}
     */
    constructor(options: LkrOptions);
    /**
     * Get the fully qualified key.
     *
     * @private
     *
     * @param  {String}  key  The key
     *
     * @return {String}
     */
    private getKey(key);
    /**
     * Add a new item to storage even if it already exists.
     *
     * @param  {String|Function|Object}  key    The key to add
     * @param  {Mixed}                   val    The value to add
     * @param  {Mixed}                   [def]  The default to pass to function if doesn't already exist
     *
     * @throws {Error}  If a key or value is not provided
     *
     * @return {Lkr}
     */
    put(key: any, val: any, def?: any): this;
    /**
     * Add an item to storage if it doesn't already exist.
     *
     * @param  {String|Function|Object}  key      The key to add
     * @param  {Mixed}                   value    The value to add
     * @param  {Mixed}                   [def]    The default to pass to function if doesn't already exist
     *
     * @return {Boolean}  Whether the item was added or not
     */
    add(key: any, value: any, def?: any): boolean;
    /**
     * Retrieve the specified item from storage.
     *
     * @param  {String|Array|Function}  key    The key to get
     * @param  {Mixed}                  [def]  The default value if it does not exist
     *
     * @return {Mixed}
     */
    get(key: any, def?: any): any;
    /**
     * Determine whether the item exists in storage.
     *
     * @param  {String|Function}  key  The key to remove
     *
     * @return {Boolean}
     */
    has(key: any): boolean;
    /**
     * Remove specified item(s) from storage.
     *
     * @param  {String|Array|Function}  key  The key or array of keys to remove
     *
     * @return {Lkr}
     */
    forget(key: any): this;
    /**
     * Retrieve the specified item from storage and then remove it
     *
     * @param  {String|Array}  key    The key to pull from storage
     * @param  {Mixed}         [def]  The default value if it does not exist
     *
     * @return {Mixed}
     */
    pull(key: any, def: any): any;
    /**
     * Get all the items in storage within the currently set namespace/driver.
     *
     * @return  {Object}
     */
    all(): object;
    /**
     * Iterate through the items within the current namespace/driver and execute the callback.
     *
     * @param   {Function}  callback            The callback function
     * @param   {Object}    [thisContext=this]  The context of this keyword
     *
     * @return  {void}
     */
    each(callback: Function, thisContext?: this): void;
    /**
     * Get the storage keys as an array.
     *
     * @return {Array}
     */
    keys(): string[];
    /**
     * Remove all items set within the current namespace/driver.
     *
     * @return {Lkr}
     */
    clean(): this;
    /**
     * Empty the current storage driver completely. Careful now.
     *
     * @return {Lkr}
     */
    empty(): this;
    /**
     * Get the total number of items within the current namespace/driver.
     *
     * @return {Integer}
     */
    count(): number;
    /**
     * Set the driver by key.
     *
     * @param   {String}  driver  The driver key
     *
     * @return  {Lkr}
     */
    driver(driver: string): Lkr;
    /**
     * Set the namespace.
     *
     * @param   {String}  namespace  The namespace
     *
     * @return  {Lkr}
     */
    namespace(namespace: string): Lkr;
    /**
     * Create a new instance of Lkr.
     *
     * @param   {Object}  options  The configuration options
     *
     * @return  {Lkr}
     */
    static make(options: LkrOptions): Lkr;
}
export default Lkr;
